/*
 * TipCalcScreen.java
 * A Simple app to generate tip and split the bill.
 */

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import java.lang.String.*;
import net.rim.device.api.system.ApplicationDescriptor;

public class TipCalcScreen extends MainScreen {
    private BasicEditField billAmountEditField = new BasicEditField("Bill Amount: $","",6,BasicEditField.FILTER_REAL_NUMERIC);
    private BasicEditField taxPercentEditField = new BasicEditField("Tax Percentage: ", "0", 6, BasicEditField.FILTER_REAL_NUMERIC);
    private BasicEditField tipPercentEditField = new BasicEditField("Tip Percentage: ", "15",3,BasicEditField.FILTER_REAL_NUMERIC);
    private BasicEditField numDinersEditField = new BasicEditField("Number of Diners: ", "1",2,BasicEditField.FILTER_REAL_NUMERIC);
    private ButtonField calcButton = new ButtonField("Calculate", ButtonField.CONSUME_CLICK);
    private ButtonField resetButton = new ButtonField("Reset", ButtonField.CONSUME_CLICK);
    private ButtonField exitButton = new ButtonField("Exit", ButtonField.CONSUME_CLICK);
    private ButtonField aboutButton = new ButtonField("About", ButtonField.CONSUME_CLICK);
    private BasicEditField tipAmountEditField = new BasicEditField("Tip Amount: $", "");
    private BasicEditField tipPerDinerEditField = new BasicEditField("Tip Per Diner: $", "");
    private BasicEditField billTotalEditField = new BasicEditField("Bill Total: $", "");
    private BasicEditField perDinerEditField = new BasicEditField("Amount per Diner: $", "");
       
    public TipCalcScreen() {
        setTitle("TipCalcBB");
        add(billAmountEditField);
        add(taxPercentEditField);
        add(tipPercentEditField);
        add(numDinersEditField);
        add(new SeparatorField());

        FieldListener calcListener = new FieldListener();
        calcButton.setChangeListener(calcListener);
        resetButton.setChangeListener(calcListener);
        exitButton.setChangeListener(calcListener);
        aboutButton.setChangeListener(calcListener);
        
        //create horizontal field manager
        HorizontalFieldManager hfm = new HorizontalFieldManager();
        hfm.add(calcButton);
        hfm.add(resetButton);
        //hfm.add(aboutButton);
        hfm.add(exitButton);
        
        add(hfm);
        add(new SeparatorField());
        add(tipAmountEditField);
        add(billTotalEditField);
        add(tipPerDinerEditField);
        add(perDinerEditField);
        add(new SeparatorField());
        }
        
    class FieldListener implements FieldChangeListener {
        public void fieldChanged(Field field, int context) {
            ButtonField button = (ButtonField) field;
                if(button.getLabel()=="Calculate") { 
                    calcPressed();
                }
                if(button.getLabel()=="Reset") {
                    resetPressed();
                }
                if(button.getLabel()=="Exit") { 
                    onClose();
                }
                if(button.getLabel()=="About") {                  
                    aboutPressed();
                }
           }
        }
    
    public void calcPressed() {
        // Convert Edit Field contents to strings
        String billAmountString = billAmountEditField.getText();
        String taxPercentString = taxPercentEditField.getText();
        String tipPercentString = tipPercentEditField.getText();
        String numDinersString = numDinersEditField.getText();
        
        if (billAmountString.equals("")) {
            Dialog.alert("You must enter a value for Bill Amount.");
            return;
        }
        
        // Parse strings to get doubles
        double billAmount = (Double.parseDouble(billAmountString));
        double taxPercent = (Double.parseDouble(taxPercentString));
        double tipPercent = (Double.parseDouble(tipPercentString));
        double numDiners = (Double.parseDouble(numDinersString));

        // Verify user input and display message box for error conditions
        if(billAmount < 0) {
            Dialog.alert("Bill Amount must be greater than zero.");
            return;
        }
        if(tipPercent < 0) {
            Dialog.alert("Tip Percentage must be zero or greater.");
            return;           
        }
        if(numDiners < 1 ) {
            Dialog.alert("Number of Diners must be greater than zero.");
            return;
        }  
        if(taxPercent < 0) {
            Dialog.alert("Tax Percentage cannot be less than zero.");
        }

        // Generate data from input fields
        double tipAmount = (billAmount * (tipPercent / 100));
        double taxAmount = billAmount * (taxPercent / 100);
        double billTotal = billAmount + tipAmount + taxAmount;
        double perDiner = billTotal / numDiners;
        double tipPerDiner = tipAmount / numDiners;
        
        // Convert doubles back to string
        String tipAmountString = Double.toString(tipAmount);
        String billTotalString = Double.toString(billTotal);
        String perDinerString = Double.toString(perDiner);
        String tipPerDinerString = Double.toString(tipPerDiner);
        
        String tipAmountFormatted = reformat(tipAmountString);
        String billTotalFormatted = reformat(billTotalString);
        String perDinerFormatted = reformat(perDinerString);
        String tipPerDinerFormatted = reformat(tipPerDinerString);
        
        // Send strings to edit fields.
        tipAmountEditField.setText(tipAmountFormatted);
        billTotalEditField.setText(billTotalFormatted);
        perDinerEditField.setText(perDinerFormatted);        
        tipPerDinerEditField.setText(tipPerDinerFormatted);  
    }
    
    public void resetPressed() {
        billAmountEditField.setText("");
        tipPercentEditField.setText("15");
        numDinersEditField.setText("1");
        tipAmountEditField.setText("");
        billTotalEditField.setText("");
        perDinerEditField.setText("");        
        tipPerDinerEditField.setText("");
    }
        
    public String reformat(String s) {
        double asDouble = (Double.parseDouble(s));
        int intPortion = (int)asDouble;
        double remainder = ((asDouble % 1) * 100);  // get remainder as double
        if((remainder % 1) * 100 > 49) {
            remainder = remainder + 1;
        }                
        int intRemainder = (int)remainder;
        String newString;
        if(intRemainder < 10) {            
            newString = Integer.toString(intPortion) + "." + "0" + Integer.toString(intRemainder);
        }        
        else {
            newString = Integer.toString(intPortion) + "." + Integer.toString(intRemainder);
        }
        return(newString);        
    }    
    
    public void aboutPressed() {
        Dialog.alert("TipCalcBB v1.1.3\n(c)2009 Red Rover\n\nGNU GENERAL PUBLIC LICENSE\nVersion 3");
    }
    
    public boolean onClose() {
        System.exit(0);
        return true;
    }
} // end class TipCalcScreen
